package cn.newfeifan.mall.module.distri.service.duser;

import cn.newfeifan.mall.framework.mybatis.core.query.LambdaQueryWrapperX;
import cn.newfeifan.mall.module.distri.controller.admin.integral.vo.IntegralSaveReqVO;
import cn.newfeifan.mall.module.distri.controller.admin.sharepath.vo.SharePathSaveReqVO;
import cn.newfeifan.mall.module.distri.controller.admin.socialstatus.vo.SocialStatusSaveReqVO;
import cn.newfeifan.mall.module.distri.dal.dataobject.integral.IntegralDO;
import cn.newfeifan.mall.module.distri.dal.dataobject.socialstatus.SocialStatusDO;
import cn.newfeifan.mall.module.distri.service.integral.IntegralService;
import cn.newfeifan.mall.module.distri.service.socialstatus.SocialStatusService;
import cn.newfeifan.mall.module.member.dal.dataobject.user.MemberUserDO;
import cn.newfeifan.mall.module.member.service.user.MemberUserService;
import cn.newfeifan.mall.module.system.service.member.MemberService;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import org.springframework.stereotype.Service;

import javax.annotation.Resource;

import org.springframework.validation.annotation.Validated;
import org.springframework.transaction.annotation.Transactional;

import java.util.*;

import cn.newfeifan.mall.module.distri.controller.admin.duser.vo.*;
import cn.newfeifan.mall.module.distri.dal.dataobject.duser.DuserDO;
import cn.newfeifan.mall.framework.common.pojo.PageResult;
import cn.newfeifan.mall.framework.common.pojo.PageParam;
import cn.newfeifan.mall.framework.common.util.object.BeanUtils;

import cn.newfeifan.mall.module.distri.dal.mysql.duser.DuserMapper;

import static cn.newfeifan.mall.framework.common.exception.util.ServiceExceptionUtil.exception;
import static cn.newfeifan.mall.module.distri.enums.ErrorCodeConstants.*;

/**
 * 推荐用户 Service 实现类
 *
 * @author 非繁人
 */
@Service
@Validated
public class DuserServiceImpl implements DuserService {

    @Resource
    private DuserMapper duserMapper;

    @Resource
    private MemberUserService memberUserService;


    @Override
    public Long createDuser(DuserSaveReqVO createReqVO) {
        // 插入
        DuserDO duser = BeanUtils.toBean(createReqVO, DuserDO.class);
        duserMapper.insert(duser);
        // 返回
        return duser.getId();
    }

    @Override
    public void updateDuser(DuserSaveReqVO updateReqVO) {
        // 校验存在
        validateDuserExists(updateReqVO.getId());
        // 更新
        DuserDO updateObj = BeanUtils.toBean(updateReqVO, DuserDO.class);
        duserMapper.updateById(updateObj);
    }

    @Override
    public void deleteDuser(Long id) {
        // 校验存在
        validateDuserExists(id);
        // 删除
        duserMapper.deleteById(id);
    }

    private void validateDuserExists(Long id) {
        if (duserMapper.selectById(id) == null) {
            throw exception(DUSER_NOT_EXISTS);
        }
    }

    @Override
    public DuserDO getDuser(Long id) {
        return duserMapper.selectById(id);
    }

    @Override
    public PageResult<DuserDO> getDuserPage(DuserPageReqVO pageReqVO) {
        return duserMapper.selectPage(pageReqVO);
    }

    @Resource
    private IntegralService integralService;

    @Resource
    private SocialStatusService socialStatusService;

    @Override
    public void hasUserAndCreat(SharePathSaveReqVO saveReqVO) {
        checkAndCreat(saveReqVO.getDescendant());
        checkAndCreat(saveReqVO.getAncestor());
    }

    private void checkAndCreat(Long userId) {
        DuserDO duserDO = duserMapper.selectOne(new LambdaQueryWrapperX<DuserDO>().eqIfPresent(DuserDO::getUserId, userId));
        MemberUserDO user = memberUserService.getUser(userId);
        // 判断是否有钱包
        IntegralDO integralDO = integralService.selectByUser(userId);
        if (integralDO == null) {
            // 创建一个钱包
            IntegralSaveReqVO integralSaveReqVO = IntegralSaveReqVO.builder()
                    .currentQuota(0)
                    .freezeQuota(0)
                    .categoryId(3L)
                    .userId(userId).build();
            Long integralId = integralService.createIntegral(integralSaveReqVO);
            integralDO = new IntegralDO().setId(integralId);
        }
        if (duserDO == null) {
            // 创建一个新的营销用户
            DuserSaveReqVO duserSaveReqVO = DuserSaveReqVO.builder()
                    .userId(userId)
                    .mobile(user.getMobile())
                    .name(user.getName())
                    .integralId(integralDO.getId())
                    .source(userId)
                    .socialStatus(1L)
                    .socialStatusPoint(0L)
                    .build();
            duserMapper.insert(BeanUtils.toBean(duserSaveReqVO, DuserDO.class));
        }
    }

}